#include <stdio.h>
#include <atomic>
#include <mutex>

#include "lo/lo.h"
#include "RtAudio.h"

using namespace std;

typedef struct {
    double value;
    string stringValue;

    double newValue;
    mutex lock;
} userData_t;

void err_handler(int num, const char *msg, const char *where)
{
    printf("Server error %i: %s in %s.\n", num, msg, where);
}

int level_cb(const char *path, const char *types,
              lo_arg **argv, int argc, lo_message msg, void *user_data)
{
    userData_t *ud = (userData_t *) user_data;
    printf("%s - %s -> %f", path, types, argv[0]->f);
    ud->lock.lock();
    ud->newValue = argv[0]->f;
    ud->lock.unlock();

//    int i;

//    printf("%s %s", path, types);

//    for (i = 0; i < argc; i++) {
//        printf(" ");
//        lo_arg_pp((lo_type) types[i], argv[i]);
//    }
//    printf("\n");

    return 0;
}

int audioCallback( void *outputBuffer, void *inputBuffer,
                   unsigned int nFrames,
                   double streamTime,
                   RtAudioStreamStatus status,
                   void *userData)
{
    userData_t *ud = (userData_t *) userData;
    if (ud->lock.try_lock()) {
        ud->value = ud->newValue;
        ud->lock.unlock();
    }
    float *in = (float *) inputBuffer;
    float *out = (float *) outputBuffer;
    for(int i = 0; i < nFrames; i++) {
        out[i] = in[i] * ud->value;
    }

    return 0;
}

int main(void)
{
    userData_t ud;
    ud.value = 0.5;

    lo_server_thread osc_thread = lo_server_thread_new("8000", err_handler);

    if (!osc_thread) {
//        printf("Error creating server thread. Port busy?.\n");
        return -1;
    }
    lo_server_thread_add_method(osc_thread, "/level", "f",
                                level_cb, &ud);
//    lo_server_thread_add_method(osc_thread, NULL, NULL ,
//                                level_cb, NULL);

    lo_server_thread_start(osc_thread);

    RtAudio audio;
    RtAudio::StreamParameters inparams;
    inparams.deviceId = audio.getDefaultInputDevice();
    inparams.nChannels = 2;
    RtAudio::StreamParameters outparams;
    outparams.deviceId = audio.getDefaultOutputDevice();
    outparams.nChannels = 2;
    RtAudioFormat format = RTAUDIO_FLOAT32;

    unsigned int bufferFrames = 64;
    audio.openStream(&outparams, &inparams, format, 44100,
                     &bufferFrames, audioCallback, &ud, NULL);

    audio.startStream();



    int i = getchar();

    audio.stopStream();
    audio.closeStream();

    lo_server_thread_stop(osc_thread);
    lo_server_free(osc_thread);
    printf("Hello World!\n");
    return 0;
}

